#include <gccore.h>
#include <wiiuse/wpad.h>
#include <wiisocket.h>
#include <unistd.h>

#include "nand.h"
#include "patcher.h"

static void* xfb = NULL;
static GXRModeObj* rmode = NULL;

int main(void) {
    VIDEO_Init();

    rmode = VIDEO_GetPreferredMode(NULL);
    xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
    console_init(xfb, 20, 20, rmode->fbWidth, rmode->xfbHeight, rmode->fbWidth * VI_DISPLAY_PIX_SZ);
    VIDEO_Configure(rmode);
    VIDEO_SetNextFramebuffer(xfb);
    VIDEO_SetBlack(FALSE);
    VIDEO_Flush();
    VIDEO_WaitVSync();
    if (rmode->viTVMode & VI_NON_INTERLACE) VIDEO_WaitVSync();

    // Init network
    wiisocket_init();

	char version_major = 1;
	char version_minor = 4;
    char version_patch = 0;

	printf("\n:---------------------------------------------------------:\n");
	printf("  RiiConnect24 Mail Patcher - (C) RiiConnect24 ");
	#ifdef COMMITHASH
		printf("v%u.%u.%u-%s\n", version_major, version_minor, version_patch, COMMITHASH);
	#else
		printf("v%u.%u.%u\n", version_major, version_minor, version_patch);
        #endif
        printf("  Compiled on %s at %s\n", __DATE__ , __TIME__);
	printf(":---------------------------------------------------------:\n\n");

	printf("Initializing... ");
    WPAD_Init();
	NAND_Init();
		printf("OK!\n");
	

    if (isDolphin()) {
        printf("\n:---------------------------------------------------------------:\n"
               ": Dolphin is not supported!                                     :\n"
               ": This tool can only run on a real Wii Console.                 :\n"
               ":                                                               :\n"
               ": Exiting in 10 seconds...                                       :\n"
               ":---------------------------------------------------------------:\n");
        sleep(10);
        exit(0);
    } else if (CheckvWii()){
        printf("\n:---------------------------------------------------------------:\n"
               ": vWii Detected                                                 :\n"
               ": This tool will still patch your nwc24msg.cfg, but you will be :\n"
               ": unable to fully utilize Wii Mail.                             :\n" 
               ":---------------------------------------------------------------:\n");
    }
    printf("\nPatching...\n\n");

    s64 friendCode = getFriendCode();
    s32 error = patchMail();

    switch (error) {
    case RESPONSE_AREGISTERED:
        printf("You have already patched your Wii to use Wii Mail.\n");
        printf("In most cases, there is no need to run this patcher again.\n");
        printf("If you're having any sorts of problems, reinstalling RiiConnect24 is unnecessary\n");
        printf("and unlikely to fix issues.\n");
        printf("If you still need to have your Wii Number removed, please contact us.\n");
        printf("\nContact info:\n- Discord: https://discord.gg/rc24\n		Wait time: Short, send a Direct Message to a developer.\n- E-Mail: support@riiconnect24.net\n		Wait time: up to 24 hours, sometimes longer\n");
        printf("\nWhen contacting, please provide a brief explanation of the issue, and");
        printf("\ninclude your Wii Number: w");
        printf("%016llu\n", friendCode);
        printf("\nPress the HOME Button to exit.\n");
        break;
    case 22:
        // cURL's error code 6 covers all the HTTP error codes higher or equal to 400
        printf("We're probably performing maintenance or having some issues. Hang tight!\n\nMake sure to check https://status.rc24.xyz/ for more info.\n\n");
        printf("\nContact info:\n- Discord: https://discord.gg/rc24\n		Wait time: Short, send a Direct Message to a developer.\n- E-Mail: support@riiconnect24.net\n		Wait time: up to 24 hours, sometimes longer\n");
        printf("\nPress the HOME Button to exit.\n");
        break;
    case 0:
        // Success
        printf("All done, all done! Thank you for installing RiiConnect24.\n");
        printf("\nPress the HOME Button to exit.\n");
        break;
    default:
        printf("There was an error while patching.\nPlease make a screenshot of this error message and contact us.\n");
        printf("\nPlease contact us using:\n- Discord: https://discord.gg/rc24\n		Wait time: Short, send a Direct Message to a developer.\n- E-Mail: support@riiconnect24.net\n		Wait time: up to 24 hours, sometimes longer\n");
        printf("\nWhen contacting, please provide a brief explanation of the issue, and");
        printf("\ninclude your Wii Number: w");
        printf("%016llu\n", friendCode);
        printf("\nPress the HOME Button to exit.\n");
        break;
    }

    while (1) {
        WPAD_ScanPads();
        u32 pressed = WPAD_ButtonsDown(0);

        if (pressed & WPAD_BUTTON_HOME) exit(0);

        VIDEO_WaitVSync();
    }

    NAND_Exit();

    return 0;
}
